@@ -65,6 +65,8 @@ class AbstractFileReader(object, metaclass=ABCMeta):
     YEAR_S_MONTH_S_DAY_HM_DATE_STRING_FORMAT = '%Y/%m/%d %H:%M'
     YEAR_S_MONTH_S_DAY_HMS_DATE_STRING_FORMAT = (
         YEAR_S_MONTH_S_DAY_HM_DATE_STRING_FORMAT + ":%S")
+    YEAR_S_MONTH_S_DAY_HMS_Z_DATE_STRING_FORMAT = (
+        YEAR_S_MONTH_S_DAY_HMS_DATE_STRING_FORMAT + " %z")
     YEAR_S_MONTH_S_DAY_HMSMS_DATE_STRING_FORMAT = (
         YEAR_S_MONTH_S_DAY_HMS_DATE_STRING_FORMAT + ".%f")
 
